==> 2.2.23.tar.gz: meshtastic-android/app/src/main/java/com/geeksville/mesh/ui/components/config/EditChannelDialog.kt <==
package com.geeksville.mesh.ui.components.config

import android.util.Base64
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.AlertDialog
import androidx.compose.material.Button
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.LocalContentAlpha
import androidx.compose.material.LocalContentColor
import androidx.compose.material.MaterialTheme
import androidx.compose.material.OutlinedTextField
import androidx.compose.material.Switch
import androidx.compose.material.Text
import androidx.compose.material.TextFieldDefaults
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.twotone.Close
import androidx.compose.material.icons.twotone.Refresh
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.geeksville.mesh.ChannelProtos
import com.geeksville.mesh.R
import com.geeksville.mesh.channelSettings
import com.geeksville.mesh.model.Channel
import com.geeksville.mesh.ui.components.EditTextPreference
import com.google.accompanist.themeadapter.appcompat.AppCompatTheme
import com.google.protobuf.ByteString
import com.google.protobuf.kotlin.toByteString
import java.security.SecureRandom

@Composable
fun EditChannelDialog(
    channelSettings: ChannelProtos.ChannelSettings,
    onAddClick: (ChannelProtos.ChannelSettings) -> Unit,
    onDismissRequest: () -> Unit,
    modifier: Modifier = Modifier,
    modemPresetName: String = "Default",
) {
    val base64Flags = Base64.URL_SAFE + Base64.NO_WRAP
    fun encodeToString(input: ByteString) =
        Base64.encodeToString(input.toByteArray() ?: ByteArray(0), base64Flags)

    var pskInput by remember { mutableStateOf(channelSettings.psk) }
    var pskString by remember(pskInput) { mutableStateOf(encodeToString(pskInput)) }
    val pskError = pskString != encodeToString(pskInput)

    var nameInput by remember { mutableStateOf(channelSettings.name) }
    var uplinkInput by remember { mutableStateOf(channelSettings.uplinkEnabled) }
    var downlinkInput by remember { mutableStateOf(channelSettings.downlinkEnabled) }

    fun getRandomKey() {
        val random = SecureRandom()
        val bytes = ByteArray(32)
        random.nextBytes(bytes)
        pskInput = ByteString.copyFrom(bytes)
    }

    AlertDialog(
        onDismissRequest = onDismissRequest,
        text = {
            AppCompatTheme {
                Column(modifier.fillMaxWidth()) {
                    var isFocused by remember { mutableStateOf(false) }
                    EditTextPreference(
                        title = stringResource(R.string.channel_name),
                        value = if (isFocused) nameInput else nameInput.ifEmpty { modemPresetName },
                        maxSize = 11, // name max_size:12
                        enabled = true,
                        isError = false,
                        keyboardOptions = KeyboardOptions.Default.copy(
                            keyboardType = KeyboardType.Text, imeAction = ImeAction.Done
                        ),
                        keyboardActions = KeyboardActions(onDone = { }),
                        onValueChanged = {
                            nameInput = it
                            if (pskInput == Channel.default.settings.psk) getRandomKey()
                        },
                        onFocusChanged = { isFocused = it.isFocused },
                    )

                    OutlinedTextField(
                        value = pskString,
                        onValueChange = {
                            try {
                                pskString = it // empty (no crypto), 128 or 256 bit only
                                val decoded = Base64.decode(it, base64Flags).toByteString()
                                val fullPsk = Channel(channelSettings { psk = decoded }).psk
                                if (fullPsk.size() in setOf(0, 16, 32)) pskInput = decoded
                            } catch (ex: Throwable) {
                                // Base64 decode failed, pskError true
                            }
                        },
                        modifier = modifier.fillMaxWidth(),
                        enabled = true,
                        label = { Text("PSK") },
                        isError = pskError,
                        keyboardOptions = KeyboardOptions.Default.copy(
                            keyboardType = KeyboardType.Password, imeAction = ImeAction.Done
                        ),
                        keyboardActions = KeyboardActions(onDone = { }),
                        trailingIcon = {
                            IconButton(
                                onClick = {
                                    if (pskError) {
                                        pskInput = channelSettings.psk
                                        pskString = encodeToString(pskInput)
                                    } else getRandomKey()
                                }
                            ) {
                                Icon(
                                    if (pskError) Icons.TwoTone.Close else Icons.TwoTone.Refresh,
                                    contentDescription = stringResource(R.string.reset),
                                    tint = if (pskError) MaterialTheme.colors.error
                                    else LocalContentColor.current.copy(alpha = LocalContentAlpha.current)
                                )
                            }
                        },
                        colors = TextFieldDefaults.outlinedTextFieldColors(
                            cursorColor = MaterialTheme.colors.onPrimary,
                        ),
                    )

                    Row(verticalAlignment = Alignment.CenterVertically) {
                        Text("Uplink enabled", // TODO move to resource strings
                            modifier = modifier.weight(1f)
                        )
                        Switch(
                            checked = uplinkInput,
                            onCheckedChange = { uplinkInput = it },
                        )
                    }

                    Row(verticalAlignment = Alignment.CenterVertically) {
                        Text("Downlink enabled", // TODO move to resource strings
                            modifier = modifier.weight(1f)
                        )
                        Switch(
                            checked = downlinkInput,
                            onCheckedChange = { downlinkInput = it },
                        )
                    }
                }
            }
        },
        buttons = {
            Row(
                modifier = modifier.fillMaxWidth(),
                verticalAlignment = Alignment.CenterVertically,
            ) {
                Button(
                    modifier = modifier
                        .fillMaxWidth()
                        .padding(start = 24.dp)
                        .weight(1f),
                    onClick = onDismissRequest
                ) { Text(stringResource(R.string.cancel)) }
                Button(
                    modifier = modifier
                        .fillMaxWidth()
                        .padding(end = 24.dp)
                        .weight(1f),
                    onClick = {
                        onAddClick(channelSettings {
                            psk = pskInput
                            name = nameInput.trim()
                            uplinkEnabled = uplinkInput
                            downlinkEnabled = downlinkInput
                        })
                    },
                    enabled = !pskError,
                ) { Text(stringResource(R.string.save)) }
            }
        }
    )
}

@Preview(showBackground = true)
@Composable
private fun EditChannelDialogPreview() {
    EditChannelDialog(
        channelSettings = channelSettings {
            psk = Channel.default.settings.psk
            name = Channel.default.name
        },
        onAddClick = { },
        onDismissRequest = { },
    )
}

==> 2.2.24.tar.gz: meshtastic-android/app/src/main/java/com/geeksville/mesh/ui/components/config/EditChannelDialog.kt <==
package com.geeksville.mesh.ui.components.config

import android.util.Base64
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.AlertDialog
import androidx.compose.material.Button
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.LocalContentAlpha
import androidx.compose.material.LocalContentColor
import androidx.compose.material.MaterialTheme
import androidx.compose.material.OutlinedTextField
import androidx.compose.material.Switch
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.twotone.Close
import androidx.compose.material.icons.twotone.Refresh
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.geeksville.mesh.ChannelProtos
import com.geeksville.mesh.R
import com.geeksville.mesh.channelSettings
import com.geeksville.mesh.copy
import com.geeksville.mesh.model.Channel
import com.geeksville.mesh.ui.components.DropDownPreference
import com.geeksville.mesh.ui.components.EditTextPreference
import com.google.accompanist.themeadapter.appcompat.AppCompatTheme
import com.google.protobuf.ByteString
import com.google.protobuf.kotlin.toByteString
import java.security.SecureRandom

private enum class PositionPrecision(val value: Int) {
    HIGH_PRECISION(32),
    MED_PRECISION(16),
    LOW_PRECISION(11),
    DISABLED(0),
    ;
}

@Composable
fun EditChannelDialog(
    channelSettings: ChannelProtos.ChannelSettings,
    onAddClick: (ChannelProtos.ChannelSettings) -> Unit,
    onDismissRequest: () -> Unit,
    modifier: Modifier = Modifier,
    modemPresetName: String = "Default",
) {
    val base64Flags = Base64.URL_SAFE + Base64.NO_WRAP
    fun encodeToString(input: ByteString) =
        Base64.encodeToString(input.toByteArray() ?: ByteArray(0), base64Flags)

    var channelInput by remember(channelSettings) { mutableStateOf(channelSettings) }
    var pskString by remember(channelInput) { mutableStateOf(encodeToString(channelInput.psk)) }
    val pskError = pskString != encodeToString(channelInput.psk)
    val useDropDown = PositionPrecision.entries
        .any { it.value == channelInput.moduleSettings.positionPrecision }

    fun getRandomKey() {
        val random = SecureRandom()
        val bytes = ByteArray(32)
        random.nextBytes(bytes)
        channelInput = channelInput.copy { psk = ByteString.copyFrom(bytes) }
    }

    AlertDialog(
        onDismissRequest = onDismissRequest,
        text = {
            AppCompatTheme {
                Column(modifier.fillMaxWidth()) {
                    val focusManager = LocalFocusManager.current
                    var isFocused by remember { mutableStateOf(false) }
                    EditTextPreference(
                        title = stringResource(R.string.channel_name),
                        value = if (isFocused) channelInput.name else channelInput.name.ifEmpty { modemPresetName },
                        maxSize = 11, // name max_size:12
                        enabled = true,
                        isError = false,
                        keyboardOptions = KeyboardOptions.Default.copy(
                            keyboardType = KeyboardType.Text, imeAction = ImeAction.Done
                        ),
                        keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() }),
                        onValueChanged = {
                            channelInput = channelInput.copy { name = it }
                            if (channelInput.psk == Channel.default.settings.psk) getRandomKey()
                        },
                        onFocusChanged = { isFocused = it.isFocused },
                    )

                    OutlinedTextField(
                        value = pskString,
                        onValueChange = {
                            try {
                                pskString = it // empty (no crypto), 128 or 256 bit only
                                val decoded = Base64.decode(it, base64Flags).toByteString()
                                val fullPsk = Channel(channelSettings { psk = decoded }).psk
                                if (fullPsk.size() in setOf(0, 16, 32)) {
                                    channelInput = channelInput.copy { psk = decoded }
                                }
                            } catch (ex: Throwable) {
                                // Base64 decode failed, pskError true
                            }
                        },
                        modifier = modifier.fillMaxWidth(),
                        enabled = true,
                        label = { Text("PSK") },
                        isError = pskError,
                        keyboardOptions = KeyboardOptions.Default.copy(
                            keyboardType = KeyboardType.Password, imeAction = ImeAction.Done
                        ),
                        keyboardActions = KeyboardActions(onDone = { }),
                        trailingIcon = {
                            IconButton(
                                onClick = {
                                    if (pskError) {
                                        channelInput = channelInput.copy { psk = channelSettings.psk }
                                        pskString = encodeToString(channelInput.psk)
                                    } else getRandomKey()
                                }
                            ) {
                                Icon(
                                    if (pskError) Icons.TwoTone.Close else Icons.TwoTone.Refresh,
                                    contentDescription = stringResource(R.string.reset),
                                    tint = if (pskError) MaterialTheme.colors.error
                                    else LocalContentColor.current.copy(alpha = LocalContentAlpha.current)
                                )
                            }
                        },
                    )

                    Row(verticalAlignment = Alignment.CenterVertically) {
                        Text("Uplink enabled", // TODO move to resource strings
                            modifier = modifier.weight(1f)
                        )
                        Switch(
                            checked = channelInput.uplinkEnabled,
                            onCheckedChange = {
                                channelInput = channelInput.copy { uplinkEnabled = it }
                            },
                        )
                    }

                    Row(verticalAlignment = Alignment.CenterVertically) {
                        Text("Downlink enabled", // TODO move to resource strings
                            modifier = modifier.weight(1f)
                        )
                        Switch(
                            checked = channelInput.downlinkEnabled,
                            onCheckedChange = {
                                channelInput = channelInput.copy { downlinkEnabled = it }
                            },
                        )
                    }

                    if (useDropDown) DropDownPreference(
                        title = "Position",
                        enabled = true,
                        items = PositionPrecision.entries.map { it.value to it.name },
                        selectedItem = channelInput.moduleSettings.positionPrecision,
                        onItemSelected = {
                            val module = channelInput.moduleSettings.copy { positionPrecision = it }
                            channelInput = channelInput.copy { moduleSettings = module }
                        }
                    ) else EditTextPreference(
                        title = "Position precision",
                        value = channelInput.moduleSettings.positionPrecision,
                        enabled = true,
                        keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() }),
                        onValueChanged = {
                            val module = channelInput.moduleSettings.copy { positionPrecision = it }
                            channelInput = channelInput.copy { moduleSettings = module }
                        },
                    )
                }
            }
        },
        buttons = {
            Row(
                modifier = modifier.fillMaxWidth(),
                verticalAlignment = Alignment.CenterVertically,
            ) {
                Button(
                    modifier = modifier
                        .fillMaxWidth()
                        .padding(start = 24.dp)
                        .weight(1f),
                    onClick = onDismissRequest
                ) { Text(stringResource(R.string.cancel)) }
                Button(
                    modifier = modifier
                        .fillMaxWidth()
                        .padding(end = 24.dp)
                        .weight(1f),
                    onClick = {
                        onAddClick(channelInput.copy { name = channelInput.name.trim() })
                    },
                    enabled = !pskError,
                ) { Text(stringResource(R.string.save)) }
            }
        }
    )
}

@Preview(showBackground = true)
@Composable
private fun EditChannelDialogPreview() {
    EditChannelDialog(
        channelSettings = channelSettings {
            psk = Channel.default.settings.psk
            name = Channel.default.name
        },
        onAddClick = { },
        onDismissRequest = { },
    )
}
